//
//  AppDelegate.swift
//  GPS
//
//  Created by Phidgets on 2018-02-14.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import Cocoa
import Phidget22Swift


@NSApplicationMain
class AppDelegate: NSObject, NSApplicationDelegate {

    @IBOutlet weak var window: NSWindow!
    let ch = GPS()
    @IBOutlet weak var phidgetInfoBox: PhidgetInfoBox!
    
    //GUI
    @IBOutlet weak var dataBox: NSBox!
    @IBOutlet weak var latitudeLabel: NSTextField!
    @IBOutlet weak var longitudeLabel: NSTextField!
    @IBOutlet weak var altitudeLabel: NSTextField!
    @IBOutlet weak var velocityLabel: NSTextField!
    @IBOutlet weak var headingLabel: NSTextField!
    @IBOutlet weak var gpsTimeLabel: NSTextField!
    @IBOutlet weak var gpsDateLabel: NSTextField!
    @IBOutlet weak var fixAquiredCheck: NSButton!
    
    //Error Window
    @IBOutlet weak var errorEventWindow: NSPanel!
    @IBOutlet var errorEventLog: NSTextView!
    @IBOutlet weak var errorEventCount: NSTextField!
    var errorCounter = 0
    
    //MARK: View events
    func applicationDidFinishLaunching(_ aNotification: Notification) {
        do {
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
            
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.positionChange.addHandler(positionchange_handler)
            let _ = ch.headingChange.addHandler(headingchange_handler)
            let _ = ch.positionFixStateChange.addHandler(fixchange_handler)
            
            /*
             * Please review the Phidget22 channel matching documentation for details on the device
             * and class architecture of Phidget22, and how channels are matched to device features.
             */
            
            /*
             * Specifies the serial number of the device to attach to.
             * For VINT devices, this is the hub serial number.
             *
             * The default is any device.
             */
            // try ch.setDeviceSerialNumber(<YOUR DEVICE SERIAL NUMBER>)
            
            /*
             * For VINT devices, this specifies the port the VINT device must be plugged into.
             *
             * The default is any port.
             */
            // try ch.setHubPort(0)
            
            /*
             * Specifies which channel to attach to.  It is important that the channel of
             * the device is the same class as the channel that is being opened.
             *
             * The default is any channel.
             */
            // try ch.setChannel(0)
            
            /*
             * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
             * In a normal environment this can be done automatically by enabling server discovery, which
             * will cause the client to discovery and connect to available servers.
             *
             * To force the channel to only match a network Phidget, set remote to 1.
             */
            
            // try Net.enableServerDiscovery(serverType: .deviceRemote)
            // try ch.setIsRemote(true)
            
            /*
             openCmdLine allows command line arguments to be passed to this program in order to
             configure channel matching.
             If you have modified the code and added your own channel matching, comment out
             the openCmdLine call, and simply replace it with the following line:
             */
            //try ch.open()
            
            try phidgetInfoBox.openCmdLine(phid: ch as Phidget)
            Timer.scheduledTimer(timeInterval: 0.1,
                                 target: self,
                                 selector: #selector(self.update(_:)),
                                 userInfo: nil,
                                 repeats: true)
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    @objc func update(_ timer:Timer){
        do{
            let time = try ch.getTime()
            let date = try ch.getDate()
            
            DispatchQueue.main.async {
                self.gpsDateLabel.stringValue = String(format:"%02d/%02d/%02d", date.day, date.month, date.year)
                self.gpsTimeLabel.stringValue = String(format:"%02d:%02d:%02d.%03d", time.hour,time.minute,time.second, time.millisecond)
            }
        }catch _ as PhidgetError {
            //ignore
        } catch {
            //ignore
        }
    }
    
    func applicationShouldTerminateAfterLastWindowClosed(_ sender: NSApplication) -> Bool {
        return true
    }
    
    func applicationWillTerminate(_ aNotification: Notification) {
        //Safely close example
        do{
            print("Closing")
            ch.attach.removeAllHandlers()
            ch.detach.removeAllHandlers()
            ch.error.removeAllHandlers()
            ch.positionFixStateChange.removeAllHandlers()
            ch.headingChange.removeAllHandlers()
            ch.positionChange.removeAllHandlers()
            try ch.close()
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event handlers
    func attach_handler(sender: Phidget){
        DispatchQueue.main.async {
            self.phidgetInfoBox.fillPhidgetInfo(device: sender)
            self.dataBox.isHidden = false
            
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.maxSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
        }
    }
    
    func detach_handler(sender: Phidget){
        DispatchQueue.main.async {
            self.phidgetInfoBox.fillPhidgetInfo(device: nil)
            self.dataBox.isHidden = true
            
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        outputError(errorDescription: data.description, errorCode: data.code.rawValue)
    }
    
    func positionchange_handler(sender: GPS, data: (latitude: Double, longitude: Double, altitude: Double)){
        DispatchQueue.main.async {
            self.latitudeLabel.stringValue = String(format:"%0.3fº",data.latitude)
            self.longitudeLabel.stringValue = String(format:"%0.3fº",data.longitude)
            self.altitudeLabel.stringValue = String(format:"%0.3fm",data.altitude)
        }
    }
    
    func headingchange_handler(sender: GPS, data: (heading: Double, velocity: Double)){
        DispatchQueue.main.async {
            self.headingLabel.stringValue = String(format:"%.1fº",data.heading)
            self.velocityLabel.stringValue = String(format:"%.1f km/h",data.velocity)
        }
    }
    
    func fixchange_handler(sender: GPS, positionFixState: Bool){
        DispatchQueue.main.async {
            if(positionFixState){
                self.fixAquiredCheck.state = NSControl.StateValue.on
            }
            else{
                self.fixAquiredCheck.state = NSControl.StateValue.off
                self.longitudeLabel.stringValue = ""
                self.latitudeLabel.stringValue = ""
                self.altitudeLabel.stringValue = ""
                self.headingLabel.stringValue = ""
                self.velocityLabel.stringValue = ""
            }
        }
    }
    
    //MARK: Error
    func outputError(errorDescription:String, errorCode:UInt32){
        DispatchQueue.main.async {
            self.errorCounter += 1
            let outputString = NSAttributedString(string: String(format:"Error %d: %@\n", errorCode, errorDescription))
            self.errorEventLog.textStorage?.beginEditing()
            self.errorEventLog.textStorage?.append(outputString)
            self.errorEventLog.textStorage?.endEditing()
            self.errorEventCount.stringValue = "\(self.errorCounter)"
            if(!self.errorEventWindow.isVisible){
                self.errorEventWindow.setIsVisible(true)
                self.errorEventWindow.center()
            }
        }
        
    }
    
    @IBAction func clearError(_ sender: Any) {
        errorEventLog.textStorage?.setAttributedString(NSAttributedString(string: ""))
        errorCounter = 0
        errorEventCount.stringValue = "0"
    }
    
    @IBAction func unwantedClick(_ sender: Any) {
        let check = sender as! NSButton
        DispatchQueue.main.async {
            check.state = (check.state == NSControl.StateValue.on) ? NSControl.StateValue.off : NSControl.StateValue.on
        }
    }
    

}

